//////////////
// Pagination
/////////////

//#include "pagination.h"

// Zone d'implantation de la table des zones
#define MEM_ZNTABLE 0x6000
// Nombre de pages alloues pour la table des zones - 4 = 64 megs de RAM
#define MEM_NBZNTABLE 4

// Zone d'implantation du rpertoire
#define MEM_REP 0Xa000

// Zone d'implantation de la premire table des pages
#define MEM_PGTABLE 0xb000

// Zone d'implantation du code du kernel
#define MEM_KERNEL 0xe000

// Zone d'implantation pagine du code du kernel
#define PAG_KERNEL 0xe000

// Nombre de pages pour le kernel - 44 = 176 k
#define NB_PGKERNEL 44

// Macros de bidouillage de mmoire

// Rcupre le numro de la page
#define GETNB(addr) addr/0x1000

// Donne l'octet correspondant  la table
#define GETOCT(nb) nb/8
// Donne le bit correspondant  la table
#define GETBIT(nb) nb%8
// Met le bit nn de o  1
#define SETBIT(o,n) o|=(1<<n)
// Met le bit nn de o  0
#define NULBIT(o,n) o&=~(1<<n)

// Sauvegarde la zone correspondant  Addr
#define SAVEZONE(Addr) SETBIT(*(char*)(MEM_ZNTABLE + GETOCT(Addr/0x1000)), GETBIT(Addr/0x1000))

// 32 bits de la page Addr -> 0b(addr sur 20 bits)000000110001
#define PAGE(Addr) ((Addr&0xfffff000)+0x31)

void ImplanteTableZones()
{
  int * pTable; // Pointeur sur le dbut de la table
  int * iTable; // Pointeur int
  char * cTable; // Pointeur char
  int Nb; // Nombre d'entiers  mettre  0
  int i; // Index

  pTable = (int*)MEM_ZNTABLE; // On lui donne d'adresse de la table des zones
  iTable = pTable;
  Nb = MEM_NBZNTABLE*0x1000/sizeof(int); // Nombre d'entiers  annuler

  // Et on met tout  0, par bouts de 4 octets
  for(i=0; i<Nb; i++)
    {
      *iTable = 0;
      iTable++;
    }

  // Ensuite, on implante la table des zones.
  // Il faut indiquer comme prises les zones de la table des zones.
  // Les zones du rpertoire des pages, et la zone d'amorce du kernel seront faites aprs
  cTable = (char *)pTable;
  
  // Les zones de la table des zones
  for(i = 0; i < MEM_NBZNTABLE; i++)
    {
      SAVEZONE(MEM_ZNTABLE + i*0x1000);
    }

  // Les zones de la mmoire vido
  for(i=0xb8000; i<0xc0000; i+=0x1000)
    {
      SAVEZONE(i);
    }

  // Les zones du kernel
  for(i=MEM_KERNEL; i<(MEM_KERNEL + NB_PGKERNEL*0x1000); i+=0x1000)
    {
      SAVEZONE(i);
    }
}

void ImplanteRepertoirePages()
{
  // On cr un rpertoire des pages vide.
  int * pTable; // Pointeur sur le dbut de la table
  int * iTable; // Pointeur int
  int i;

  pTable = (int *)MEM_REP;
  iTable = pTable;

  // RAZ de tout le rpertoire
  for(i=0; i<(0x1000/sizeof(int)); i++)
    {
      *iTable = 0;
      iTable++;
    }

  pTable = (int *)MEM_PGTABLE;
  iTable = pTable;

  // RAZ de toute la table
  for(i=0; i<(0x1000/sizeof(int)); i++)
    {
      *iTable = 0;
      iTable++;
    }

  // Sauvegarde du rpertoire et de la table dans la table des zones
  SAVEZONE(MEM_REP);
  SAVEZONE(MEM_PGTABLE);
  
  // Bon, plus qu' rajouter nos valeurs dans la table
  // Rajout de la premire table dans le rpertoire
  *((int *)MEM_REP) = (int)MEM_PGTABLE;

  // Rajout de la mmoire vido
  for(i=0; i < 8; i++)
    {
      *(((int *)MEM_PGTABLE)+i) = (int)PAGE(0xb8000+i*0x1000);
    }

  // Rajout de la table des zones
  for(i=0; i < 4; i++)
    {
      *(((int *)MEM_PGTABLE)+8+i) = (int)PAGE(0x6000+i*0x1000);
    }

  // Rajout du rpertoire des pages
  *(((int *)MEM_PGTABLE)+12) = (int)PAGE(0xc000);

  // Rajout de la premire table des pages
  *(((int *)MEM_PGTABLE)+13) = (int)PAGE(0xd000);

  // Rajout des tables du kernel
  for(i=0; i < NB_PGKERNEL; i++)
    {
      *(((int *)MEM_PGTABLE)+14+i) = (int)PAGE(PAG_KERNEL+i*0x1000);
    }
  

  // Pfiou!! Bon, a doit tre bon, maintenant.
}

void Pagination()
{
  // Bon, on commence  implanter la table des zones
  ImplanteTableZones();

  // Ensuite, on implante le rpertoire des pages
  ImplanteRepertoirePages();
}





